#!/bin/bash
#
# update package tomcat
# usage:
# 1) place latest tomcat in system.node/dev.doc/scripts/apache-tomcat-x.x.x.tar.gz
# 2) change TOMCAT variable in this script to reflect new version
# 3) add this update script to the changelog
#    'update_tomcat.sh'
#

TOMCAT="apache-tomcat-6.0.26"
CURDIR=`pwd`

if [ -f "$CURDIR/$TOMCAT.tar.gz" ]; then
	echo "found $CURDIR/$TOMCAT.tar.gz, starting update"
else
        echo "tomcat file $CURDIR/$TOMCAT.tar.gz not found, exiting"
        exit 2
fi

cd /Node

# preparation - kill the old fart
echo "stopping old tomcat"
/Node/bin/nodectl stop tomcat

# extract tomcat
echo "extracting new tomcat version $TOMCAT"
tar xzf "$CURDIR/$TOMCAT.tar.gz"

# adapt directory structure
echo "adapt new tomcat package"
cd /Node/$TOMCAT

# remove unused dirs
rm -rf conf logs shared webapps

# relink dirs
ln -s /Node/etc/tomcat/ conf
ln -s /Node/var/tomcat/logs/ logs
ln -s /Node/var/tomcat/shared/ shared
ln -s /Node/.nodejava/webapps/ webapps

# remove old symlink & generate new symlink
echo "update permissions and symlink"
cd /Node
chown -R node:node_public $TOMCAT
rm -f tomcat
ln -s $TOMCAT/ tomcat

# now clean the whole webapps dir, since Portal.Node webapp will be broken otherwise
rm -rf /Node/tomcat/webapps/*.*

# start new tomcat
/Node/bin/nodectl start tomcat
