/**
 * Spellchecker which utilizes the Google Toolbar Spellcheck API as described on
 * http://weblogs.asp.net/pwelter34/archive/2005/07/19/google-toolbar-spell-check-api.aspx
 * http://www.loresoft.com/projects/netspell/google-toolbar-spell-check-api/default.aspx
 */
var GTXSpellCheck = {
	/**
	 * language used for spellchecking
	 * eg. en, de...
	 */	
	lang: null,
	
	/**
	 * tag id of the currently activated live edit tag which will be checked
	 */
	tagId: null,
	
	/**
	 * url of spellcheck service
	 */
	spellcheckURL: null,
	
	/**
	 * reference kept for asynchronous callbacks
	 */
	ref: null,
	
	/**
	 * string to be checked for spelling errors
	 */
	checkStr: null,
	
	/**
	 * error-checked output string
	 */
	outStr: "",
	
	/**
	 * number of errors found in the string
	 */
	numErrors: null,
	
	/**
	 * this is the offset to which to string has been corrected
	 */
	lastOffset: 0,
	
	/**
	 * mark true if we're spellchecking right now
	 */
	checking: false,
	
	/**
	 * check method
	 * may be ispell or google
	 */
	spellcheckMethod: "ispell",
	
	/**
	 * icon urls
	 */
	spellcheckIcon: null,
	loadIcon: null,
	
	/**
	 * ispell umlaut conversions
	 */
	umlauts: null,
	ispellUmlauts: null,
	
	/**
	 * marked words are stored inside an array
	 */
	markedWords: new Array(),
	
	/**
	 * initialize the spellchecker with a given language
	 * @param string language to be checked against 
	 */
	init: function(stag, lang, spellcheckIcon, loadIcon) {
		GTXSpellCheck.ref = this;
		this.lang = lang;
		this.spellcheckURL = stag + "&do=1041&lang=" + lang;
		this.spellcheckIcon = spellcheckIcon;
		this.loadIcon = loadIcon;
		jQuery().ajaxError(function(event, request, settings) {
			alert("Unfortunately an error occured while querying the server.\n" + 
				request.responseText
			);
			GTXSpellCheck.ref.reset();
		});
		// only used with ispell
		if (this.spellcheckMethod == "ispell") {
			// initialize umlauts
			this.umlauts = new Array("\u00E4", "\u00F6", "\u00FC", "\u00C4", "\u00D6", "\u00DC", "\u00DF");
			this.ispellUmlauts = new Array('a"', 'o"', 'u"', 'A"', 'O"', 'U"', 'sS');
		}
	},
	
	/**
	 * check a given string
	 * @param string to be checked 
	 */
	check: function() {
		if (this.checking) {
			return;
		}
		this.checking = true;
		jQuery("#livespellcheckicon").attr("src", this.loadIcon);
		this.checkStr = this.cleanString(jQuery("#p_" + this.tagId + "_1").html());
		// ispell wants some crappy umlaut replacement which will be undone later
        if (this.spellcheckMethod == "ispell") {
            this.checkStr = this.strReplace(this.umlauts, this.ispellUmlauts, this.checkStr);
        }
		data = { string: "" };
		// we don't want the checkStr to have it's html tags replaced by underscores,
		// as the resulting error checked string is built from checkStr, and all html
		// tags would be lost
		data.string = this.html2Underscore(this.checkStr);
		jQuery.post(this.spellcheckURL, data, function(data) {
			GTXSpellCheck.ref.parseSpellcheckData(data);
		}, "xml");
	},
	
	/**
	 * emulates phps str replace function
	 */
	strReplace: function(search, replace, string) {
		for (var i = 0; i < search.length; i++) {
			string = string.replace(new RegExp(search[i], "g"), replace[i]);
		}
		return string;
	},
	
	/**
	 * parse xml data sent by the spellchecker
	 * @param xml structure 
	 */
	parseSpellcheckData: function(data) {
		// google returned an error
		if (jQuery(data).find("spellresult").attr("error") != 0) {
			alert("an error occured");
			this.reset();
			return;
		}
		this.numErrors = jQuery(data).find("c").length;
		if (this.numErrors == 0) {
			// great - no errors!
			this.reset();
			return;
		}
		
		// mark each error
		jQuery(data).find("c").each(function(i) {
			if (GTXSpellCheck.ref.spellcheckMethod == "ispell") {
				word = jQuery(this).attr("w");
				GTXSpellCheck.ref.ispellMarkError(word);
			} else if (GTXSpellCheck.ref.spellcheckMethod == "google") {
				offset = parseInt(jQuery(this).attr("o"));
				len = parseInt(jQuery(this).attr("l"));
				GTXSpellCheck.ref.googleMarkError(i, offset, len);
			}
		});
		
		// if we used ispell then umlauts have to be returned to normal chars
		if (this.spellcheckMethod == "ispell") {
			this.outStr = this.repairIspellUmlautWords(this.outStr);
			this.outStr = this.strReplace(this.ispellUmlauts, this.umlauts, this.outStr);
		}
		
		// finally apply the spellchecked string
		jQuery("#p_" + this.tagId + "_1").html(this.outStr);
		this.reset();
	},
	
	/**
	 * google specific error marks
	 * this function will build a new string which contains the errors as marked strings
	 * and can be returned
	 * @param word marked as false
	 */
	ispellMarkError: function(word) {
		if (this.outStr == "") {
			this.outStr = this.checkStr;
		}
		// check if we've already marked that word and add it to the list if not
		if (this.arrayContains(this.markedWords, word)) {
			return;
		} else {
			this.markedWords.push(word);
		}
		
		// highlight all occurences of the misspelled
		this.outStr = this.outStr.replace(new RegExp("([^a-z])" + word + "([^a-z])", "gi"), "$1<gtx:span class=\"spellcheckerror\">" + word + "</gtx:span>$2");
	},
	
	/**
	 * see if an array contains an object
	 * decreasing count as this is said to be the faster method in js... whyever...
	 * @param array to be searched for
	 * @param object to be searched for
	 * @return true if the object is found in the array 
	 */
	arrayContains: function(array, object) {
		var i = array.length;
		while (i--) {
			if (array[i] === object) {
				return true;
			}
		}
		return false;
	},
	
	/**
	 * in english words like "wrter" will be recognized as erroneous and be split like:
	 * wo (this is an error)
	 * " (some char ispell does not get upset about
	 * rter (this is also an error)
	 * these broken words will be reassembled by this function
	 */
	repairIspellUmlautWords: function(string) {
		return string.replace(new RegExp("([aouAOU])</gtx:span>\"<gtx:span class=\"spellcheckerror\">", "g"), "$1\"");
	},
		
	/**
	 * google specific error marks
	 * this function will build a new string which contains the errors as marked strings
	 * and can be returned
	 * @param current iteration
	 * @param error offset from start
	 * @param error length
	 */
	googleMarkError: function(iteration, offset, length) {
		// add contents from last offset position to this offset to out string
		this.outStr += this.checkStr.substr(this.lastOffset, offset - this.lastOffset);

		// mark current error
		this.outStr += "<gtx:span class=\"spellcheckerror\">";
		this.outStr += this.checkStr.substr(offset, length);
		this.outStr += "</gtx:span>";
		
		this.lastOffset = offset + length;
		
		// finish output string after adding the last error
		if ((iteration + 1) == this.numErrors) {
			this.outStr += this.checkStr.substr(this.lastOffset, this.checkStr.length - this.lastOffset);
			return;
		}
	},
	
	/**
	 * clean a string from all the spellchecking tags that have been added
	 * @param string to be cleaned
	 * @return cleaned string
	 */
	cleanString: function(string) {
		// IE adds the processing instruction automatically which might cause trouble
		string = string.replace(/<\?xml:namespace prefix = gtx \/>/gi, "");
		// also remove our spellcheck error markers
		string = string.replace(/<\/?gtx:span[^>]*>/gi, "");
		return string;
	},

	/**
	 * replace html tags by underscores so that they are not harmed by checking
	 * and left intact, like:
	 *	<strong>Hello!</strong>
	 * would turn to
	 *	________Hello!_________
	 * which will not offend the spellchecker.
	 * @param string to be treated
	 * @return string with html tags replaced by underscores
	 */
	html2Underscore: function(string) {
		// now replace all html tags by underscores to exclude them spellchecking
		htmlTagsRegex  = /(<[^>]+>)/g;
		while ((match = htmlTagsRegex.exec(string)) != null) {
			string = string.replace(match[0], this.repeatStr("_", (match[0].length - 1)));
		}
		return string;
	},
	
	/**
	 * reset the internal properties used for correction to their
	 * standard values for the next correction
	 */
	reset: function() {
		this.lastOffset = 0;
		this.outStr = "";
		this.checking = false;
		this.markedWords = new Array();
		jQuery("#livespellcheckicon").attr("src", this.spellcheckIcon);
	},
	
	/**
	 * places the icon near the liveedit field currently activated
	 * 
	 * @param string which resembles the currently active live edit field
	 */
	activate: function(tagId) {
		this.tagId = tagId;
		tPos = jQuery("#t_" + tagId).position();
		icon = jQuery("#livespellcheckicon");
		// unfortunately scrollTop() and scrollLeft() did not work as described in the jQuery 1.3 api
		icon.attr("style", "top: " + (tPos.top + 1) + "px; left: " + (tPos.left - 19) + "px;");
		icon.fadeIn("normal");
	},
	
	/**
	 * repeat a provided string n times
	 * @param string to be repeated
	 * @param number of times the string should be repeated
	 * @return string repeated n times 
	 */
	repeatStr: function(string, nTimes) {
		var i = 0;
		var ret = string;
		while (i < nTimes) {
			ret += string;
			i++;
		}
		return ret;
	}
}
