<?php
function xnl_is_instance_of($anObject, $aClass)
{
if (!is_object($anObject)) return false;
return (get_class($anObject) == strtolower($aClass) or (is_subclass_of($anObject, $aClass)));
}

/**
 *                       
 * 
 *                                             
 * 
 *             
 *                                       
 **/
function xnl_dump_syntax_tree( $tree, $prefix = "" )
{
if ( is_array( $tree ) )
{
$ret = "";
foreach ( $tree as $subtree )
$ret .= "$prefix&gt; ( " . xnl_dump_syntax_tree( $subtree, "--$prefix" ) . " ) ";
}
else if ( is_object($tree) && is_subclass_of( $tree, "XNL_Operator" ) )
{
$oStrs = $tree->getOperatorStrings();
$ret = $oStrs[0];
}
else if ( xnl_is_instance_of( $tree, "XNL_FunctionResult" ) )
{
$func = $tree->getFunction();
$fStr = $func->getFunctionStrings();
$ret =  $fStr[0] . xnl_dump_syntax_tree( $tree->getParameters(), "--$prefix" );
}
else
$ret = "$prefix&gt; ".$tree;
return $ret;
}

/**
 *                             
 * 
 *                                                      
 *                                                            
 *                                                                                       
 *                                                
 *                                
 **/
function xnl_create_expression_dump( $expression, $pos, $leftDump=1, $rightDump=2 )
{
$c = count($expression);
for ( $i = $pos - $leftDump; $i < $pos + $rightDump  &&  $i < $c; $i++ )
{
if ( $i >= 0 )
{
$ret .= xnl_dump_syntax_tree( $expression[$i] ) . " ";
}
}
return $ret;
}


function xnl_raise_parse_error( $pos, $where, $output )
{
global $XNL_SHOW_ERRORS, $DEBUG;
if ( $XNL_SHOW_ERRORS || $DEBUG["xnl"] )
{
echo "<br>Parse error in <b>$where</b> at pos <b>$pos</b>!<br>";
echo "Details: $output<br>";
}
}                   

function xnl_break_on_parse_error( $where, $comment )
{
global $DEBUG;
if ( $DEBUG["xnl"] ) echo "Function called by: <b>$where</b>.<br>";
if ( $DEBUG["xnl"] ) echo "Details: $comment<br>";
}

class XNL_FunctionResult
{
var $d_result, $d_params, $d_func;

function XNL_FunctionResult( $function, $parameters, $result )
{
$this->d_result = $result;
$this->d_params = $parameters;
$this->d_func = $function;
}

function getResult()
{
return $this->d_result;
}

function getFunction()
{
return $this->d_func;		
}

function getParameters()
{
return $this->d_params;	
}
}

/**
 *                  
 *
 *                                                             
 *  
 *                                       
 *                                                                                     
 *         
 **/
function xnl_grep_string( $expression, &$start, $quote="\"" )
{
$pos = strpos( $expression, $quote, $start );
if ( $pos !== false )
{
$prev_ch = substr( $expression, $pos-1, 1 );
if ( $prev_ch != "\\" )
{
$ret = substr($expression, $start, $pos-$start);
$start = $pos;
return $ret;
}
else
{   
$pos++; 
$ret = substr($expression, $start, $pos-$start-2) . $quote . xnl_grep_string( $expression, $pos, $quote );
$start = $pos;
return $ret;
}
}
else
{
xnl_raise_parse_error( $i, "xnl_grep_string","string has no ending" );
return false;
}
}


/**
 *                      
 * 
 *              
 *                           
 *                                    
 *                                                                                   
 **/
function xnl_parseParameters( $e_str, &$pos, $maxPos )
{
$params = array();

$ch = substr( $e_str, $pos, 1 );
while ( $ch == " " )
{
$pos++;
$ch = substr( $e_str, $pos, 1 );
}

if ( $ch == "(" )
{
$pos++;
$pEndPos = $pos;
$openBraces = 0;
$escaped = false;
$inString = false;
$stringOpener = '';
while ( true )
{
$ch = substr( $e_str, $pEndPos, 1 );

if ( !$escaped )
{
switch( $ch )
{
case "(": 
if ( !$inString )
$openBraces++;
break;
case "\\":
$escaped = true;
break;
case "\"":
if ( !$inString ) {
$inString = true;
$stringOpener = '"';
} else if ( $stringOpener == '"' ) {
$inString = false;
}
break;
case "'":
if ( !$inString ) {
$inString = true;
$stringOpener = "'";
} else if ( $stringOpener == "'" ) {
$inString = false;
}
break;
case ",":
if ( !$inString )
{
$param = substr( $e_str, $pos, $pEndPos - $pos );
if ( strlen( trim( $param ) ) )
$params[] = $param; 
$pos = $pEndPos+1;
}
break;
case ")":
if ( !$inString )
{
if ( $openBraces == 0 )
{
$param = substr( $e_str, $pos, $pEndPos - $pos );
if ( strlen( trim( $param ) ) )
$params[] = $param; 
$pos = $pEndPos+1;
return $params;
}
else
$openBraces--;
}
break;
} //       
 }
else
{
$escaped = false;            	
}
$pEndPos++;

if ( $pEndPos > $maxPos )
{
xnl_raise_parse_error( $pEndPos, "xnl_parseParameters", "unexpected end of expression");
return false;
}
}
}
else
{
xnl_raise_parse_error( $pos, "xnl_parseParameters", "( expected" );
return false;
}

}

/**
 *            
 * 
 *                                                                                           
 *                 
 *                                                                                  
 *                                                                                               
 *                                  
 * 
 *              
 *            
 *         
 **/
function xnl_parse( $e_str, &$pos, $open_braces=0 )
{
$i = $pos;
$count = strlen($e_str);
$escaped = false;
$error = false;
$str = "";
$ret = array();

while ( $i < $count && !$error )
{
$ch = substr( $e_str, $i, 1 );

if ( !$escaped )
{
//                                                                                         
 $operator = xnl_is_operator( $e_str, $i );
if ( $operator !== false )
{
if ( strlen( $str ) )
$ret[] = $str;
$ret[] = $operator; 
$str = "";
}
//                                                             
 else if ( ($function = xnl_is_function( $e_str, $i ) ) !== false )
{
$start = $i;
$fParams = xnl_parseParameters( $e_str, $i, $count );
if ( $fParams === false )
{
xnl_break_on_parse_error( "xnl_parse", "at pos: $start" );
return false;
}
$pCount = count( $fParams );
if ( $pCount < $function->getMinParamCount() )
{
$fStrs = $function->getFunctionStrings();
xnl_raise_parse_error( $i, "xnl_parse", "too few arguments to function '".
$fStrs[0] ."' ");
return false; 
}                
else if ( $pCount > $function->getMaxParamCount() )
{
$fStrs = $function->getFunctionStrings();
xnl_raise_parse_error( $i, "xnl_parse", "too many arguments to function '".
$fStrs[0] ."' ");
return false; 
}
//                                                                                            
 $eval_fParams = array();
foreach ( $fParams as $fParam )
{
$eval_Param = xnl_evaluate($fParam);
if ( $eval_Param !== false )
{
$eval_fParams[] = $eval_Param;
}
else
{
xnl_break_on_parse_error("xnl_parse", "while evaluating parameter: $fParam");
return false;                    	
}
}
//                                                                                                 
 $fresult = $function->execute( $eval_fParams );
if ( $fresult !== false )
{
$xx = new XNL_FunctionResult( $function, $eval_fParams, $fresult );
$ret[] = $xx;
}
else
{
$fStrs = $function->getFunctionStrings();
xnl_break_on_parse_error( "xnl_parse", "while executing function '".$fStrs[0]."'");
}

}
//                                                            
 else 
{
switch( $ch )
{
case "\\": 
$escaped = true;
break;
case "'":
case "\"": 
$i++;
$start = $i;
$substr = xnl_grep_string($e_str, $i, $ch);
if ( $substr === false )
{
xnl_break_on_parse_error( "xnl_parse", "at pos: $start" );
return false;
}
$ret[] = $substr;
$str = "";
break;
case " ":
if ( strlen( $str ) )
{
$ret[] = $str;
$str = "";
}
break;
case "(":
$i++;
$start = $i;
$result = xnl_parse( $e_str, $i, $open_braces+1 );
if ( $result === false )
{
xnl_break_on_parse_error( "xnl_parse", "at pos: $start" );
return false;
}
$ret[] = $result;
break;
case ")":
if ( !$open_braces )
{
xnl_raise_parse_error( $i, "xnl_parse", "closing brace without opening brace!" );
return false;
}
if  ( strlen( $str ) )
$ret[] = $str;
$pos = $i;
return $ret;
default:
$str .= $ch;
} //       
 $i++;
}
}
else
{
switch( $ch )
{
case "'":
case "\"":
case "\\";
$str .= $ch;
break;
default:
xnl_raise_parse_error( $i, "xnl_parse", "unknown escaped character: $ch" );
return false;
} //       
 $escaped = false;
$i++;
}

}
if ( strlen( $str ) )
$ret[] = $str;
$pos = $i;
return $ret;
}


/**
 *                      
 *
 *                                        
 *  
 *                                                              
 *                                                              
 *                                                           
 *                                                       
 **/
function xnl_get_operator_id( $operator )
{
if ( is_object($operator) && is_subclass_of( $operator, "XNL_LeftUnaryOperator" ) )
return 1;
else if ( is_object($operator) && is_subclass_of($operator, "XNL_BinaryOperator") )
return 2;
else
return 0;

}

/**
 *                          
 *
 *                                                                                        
 *                             
 *                                                                                          
 *                                 
 *  
 *              
 *                    
 *                       
 *         
 **/
function xnl_fetch_right_operand( $parts, $parts_count, &$pos )
{
global $DEBUG;

if ( $pos < $parts_count )
{
$roperand = xnl_eval_expression( $parts, $pos );
if ( $DEBUG["xnl"] ) echo "RIGHT: $roperand<br>";
if ( is_object($roperand) && is_subclass_of( $roperand, "XNL_Operator") )
{
xnl_raise_parse_error($pos, "xnl_fetch_right_operand", "right operand is an Operator!" );
return false;
}
else if ( xnl_is_instance_of( $roperand, "XNL_FunctionResult") )
{
return $roperand->getResult();
}
return $roperand;
} 
else
{
xnl_raise_parse_error($pos, "xnl_fetch_right_operand", "no right operand present at the end of the line!");
return false;
}
}

function xnl_parse_left_unary_operator( $parts, $parts_count, &$pos )
{
global $DEBUG;

if ( $DEBUG["xnl"] ) echo "PARSING LEFT UNARY OPERATOR...(pos: $pos)<br>";
$operator = $parts[$pos];
$pos++;
$start = $pos;
$roperand = xnl_fetch_right_operand( $parts, $parts_count, $pos );
if ( $DEBUG["xnl"] ) echo "RIGHT OPERAND IS: $roperand<br>";
if ( $roperand === false )
{
xnl_break_on_parse_error( "xnl_parse_left_unary_operator", "at pos: $start" );
return false;
}

return  $operator->performOperation( $roperand );     	
}

function xnl_parse_binary_operator( $parts, $parts_count, $my_loperand, &$pos )
{
global $DEBUG;

$i = $pos;

if ( $DEBUG["xnl"] ) echo "I: $i; count: $parts_count; pos:$pos;<br>";

if ( $DEBUG["xnl"] ) echo "LEFT: $my_loperand<br>";

$operator_id = xnl_get_operator_id( $my_loperand );

if ( !$operator_id )
{
//                                                                                               
 if ( $i < $parts_count )
{
$operator = $parts[$i];
if ( $DEBUG["xnl"] ) echo "OPERATOR: " . get_class($operator) . "<br>";

//                                                                                                  
 if ( is_subclass_of( $operator, "XNL_BinaryOperator" ) )
{
//                                                                                                       
 //                                                              
 $i++;
$start = $i;
$roperand = xnl_fetch_right_operand( $parts, $parts_count, $i );
if ( $roperand === false ) //                 
 {
xnl_break_on_parse_error("xnl_parse_binary_operator", "at pos: $start" );
return false;
}

//                                                                              
 //                                                                               
 //                                                            
 if ( $i < $parts_count )
{
$next_operator = $parts[$i];
do
{
if ( $DEBUG["xnl"] ) echo "NEXT OP: ";
if ( $DEBUG["xnl"] ) var_dump($next_operator);
$higher_priority_found = false;
if ( is_object( $next_operator ) ) //                         
 {
if ( is_subclass_of( $next_operator, "XNL_BinaryOperator" ) )
{
if ( $next_operator->getPriority() > $operator->getPriority() )
{
$higher_priority_found = true;
if ( $DEBUG["xnl"] ) echo "HIGHER PR (".$next_operator->getPriority().">".$operator->getPriority().")<br>";
$start = $i;
$roperand = xnl_parse_binary_operator( $parts, $parts_count, $roperand, $i );
if ( $roperand === false ) //              
 {
xnl_break_on_parse_error( "xnl_parse_binary_operator", $start );
return false;
}
$next_operator = $parts[$i];
}
}
}
} while ( $higher_priority_found );
}
//                                                    
 if ( $DEBUG["xnl"] ) echo "PERFORMING: LEFT=$my_loperand RIGHT=$roperand<br>";
if ( $DEBUG["xnl"] ) echo "OPERATOR IS: " . get_class($operator) . "<br>";
$result = $operator->performOperation( $my_loperand, $roperand );
if ( $DEBUG["xnl"] ) echo "RESULT: $result<br><br>";
$pos = $i;
return $result;
}
else
{
xnl_raise_parse_error($i, "xnl_parse_binary_operator", "no binary operator present: " . get_class($operator));
return false;
}
}
else
{
xnl_raise_parse_error($i, "xnl_parse_binary_operator", "no binary operator present!");
return false;
}

}
else
{
xnl_raise_parse_error( $i, "xnl_parse_binary_operator", "not a left operand!");
return false;
}   
}


/**
 *                      
 *
 *                                                                     
 *                                                                                   
 *  
 *                         
 *                                 
                                                                                          
                                                                                            
 *                                                                                                
 *                                         
 **/
function xnl_eval_expression( $expression, &$pos )
{

global $DEBUG, $XNL_SHOW_ERRORS;
if ( is_array($expression[$pos]) )
{
$count = count($expression[$pos]);
$subexpr_pos = 0;
$result = xnl_eval_expression( $expression[$pos], $subexpr_pos );
while ( $subexpr_pos < $count )
{
if ( $DEBUG["xnl"] ) echo "<br>----------RUN: $pos / $subexpr_pos--------------------------<br>";
if ( $DEBUG["xnl"] ) echo "FIRST LEFT: $result<br><br>";
$result = xnl_parse_binary_operator($expression[$pos], $count, $result, $subexpr_pos );
if ( $DEBUG["xnl"] ) echo "<br>OUTER RESULT: $result<br>";
if ( $DEBUG["xnl"] ) echo "-----------------------------------------<br><br>";
if ( $result === false )
{
$ex = xnl_create_expression_dump( $expression[$pos], $subexpr_pos, 3, 1 );
xnl_break_on_parse_error( "eval_expression", "at pos: $subexpr_pos" );
if ( $DEBUG["xnl"] || $XNL_SHOW_ERRORS )
echo "Failed expression was: " . $ex . "<br>";
$pos++;
return false;
}
}

$pos ++;
if ( xnl_is_instance_of( $result, "XNL_FunctionResult" ) )
return $result->getResult();
return $result;
}
else if ( is_object($expression[$pos]) && is_subclass_of( $expression[$pos], "XNL_LeftUnaryOperator" ) )
{
$result = xnl_parse_left_unary_operator( $expression, count($expression), $pos );
if ( $DEBUG["xnl"] ) echo "UNARY OPERATOR RETURNED: $result<br><br>";
return $result;
}
else
{
$result = $expression[$pos];
$pos++;
if ( xnl_is_instance_of( $result, "XNL_FunctionResult" ) )
{
return $result->getResult();
}
return $result;
}
}


/**
 *                        
 *
 *                                                                             
 *                                                               
 * 
 *                                       
 *         
 **/
function xnl_eval_whole_expression( $expression )
{
$pos = 0;
$expression_array = array($expression);
return xnl_eval_expression($expression_array, $pos);
}

$TREE_COLORS = array(
"000000", "DD0000", "0000DD", "00DD00"
);

function xnl_print_syntaxtree( $parts, $prefix="", $colIdx = 0 )
{
global $TREE_COLORS;

if ( $colIdx >= count($TREE_COLORS) )
$colIdx = 0;
$color = $TREE_COLORS[$colIdx];

if ( is_array( $parts ) )
{
foreach ( $parts as $key  => $dmp )
$ret .= "<font color=$color>$prefix&gt; $key:</font><br>" . xnl_print_syntaxtree( $dmp, "$prefix--", $colIdx+1 );
return $ret;
}
else if ( is_object( $parts ) )
{
$ret .= "<font color=$color>";
$c = get_class($parts);
$ret .= "$prefix&gt" . $c . ":<br>";
$vs = get_object_vars($parts);
$ret .= "</font>";
foreach ( $vs as $k=>$v )
{
$ret .= "<font color=$color>$prefix--&gt; $k: </font><br>".xnl_print_syntaxtree($v, "$prefix--", $colIdx+1)."<br>";
}
return $ret;
}
return "<font color=$color>$prefix--&gt; $k:'".$parts."'<br></font>";
}

/**
 *           
 * 
 *                                                               
 * 
 *                   
 *         
 **/
function xnl_evaluate( $str, $errorOutput = false )
{
global $DEBUG, $XNL_SHOW_ERRORS;

$XNL_SHOW_ERRORS = $errorOutput;
if ( $DEBUG["xnl"] )
{
echo "Parsing: ".htmlentities_utf8($str)."<br>";
echo "----------------------<br>";
}

$i = 0;
$parts = xnl_parse($str, $i);
if ( is_array($parts) )
{
if ( $DEBUG["xnl"] )
{
echo "___SYNTAX-TREE___<br>";
echo xnl_print_syntaxtree( $parts );
echo "<br>-----------------------------------<br><br>";
}
return xnl_eval_whole_expression( $parts );
}
else
{
if ( $DEBUG["xnl"] || $XNL_SHOW_ERRORS )
echo "Failed expression was: " . $str . "<br>";
return false;
}
}

?>
