<?

/**
 *                                                            
 *
 **/
class XNLRequest extends BaseObject {

var $config;
var $qualified;
var $packages;
var $error;
var $errorMsg;

var $loader;

var $params = array();

function XNLRequest($loader) {
//                        
 $this->clearConfigs();
$this->resetError();
$this->setLoader($loader);
}

function setLoader($loader) {
$this->resetError();
$this->loader = $loader;
$this->reloadConfig();
}

function &getLoader() {
return $this->loader;
}

/**
  *                                                  
  **/
function reloadConfig() {
$this->clearConfigs();
$packages = $this->loader->getPackages();
reset($packages);
while (list($package,$packel) = each($packages)) {
$connect = $this->loader->getConnect($package);
$functions = $this->loader->getFunctions($package);
$packel["connect"] = $connect;
reset($functions);
while (list($function, $funcel) = each($functions)) {
$full = $package.".".$function;
$this->config[$full] = $funcel;
$this->qualified[$function][] = $full;
}
$this->packages[$package] = $packel;
}
}

/**
  *                                                          
  **/
function clearConfigs() {
$this->config = array();
$this->packages = array();
$this->qualified = array();
}

/**
  *                                                                      
  *                              
  **/
function functionExists($function) {
$func = $this->getFullyQualified($function);
return is_array($this->config[$func]);
}

/**
  *                                                              
  *                                          
  *                                                                                                
  *                                          
  *                                                              
  *                                                      
  **/
function getFullyQualified($function, $strict = false) {
if ($this->isFullyQualified($function)) return $function;
if (!is_array($this->qualified[$function])) return false;
if ($strict && count($this->qualified[$function]) > 1) return false;
return $this->qualified[$function][0];
}

/**
  *                                                                                            
  *                                                                 
  *                                  
  *                                             
  **/
function isFullyQualified($function) {
return (strpos($function, ".")>0)?true:false;
}

/**
  *                                                                                       
  *                                  
  *                                            
  **/
function isAmbigous($function) {
if ($this->isFullyQualified($function)) return false;
if (is_array($this->qualified[$function]) && count($this->qualified[$function]) > 1) return true;
return false;
}

/**
  *                                           
  *                                
  *                                    
  **/
function packageExists($package) {
if (!$package) return false;
return is_array($this->packages[$package]);
}

/**
  *                                                                            
  *                                           
  *                                                           
  **/
function getFuncDescription($function) {
$func = $this->getFullyQualified($function);
if (!$this->functionExists($func)) return false;
return $this->config[$func]["description"];
}

function funcAllowCache($function) {
$func = $this->getFullyQualified($function);
if (!$this->functionExists($func)) return false;
return $this->config[$func]["cache"]["allowcache"];
}

function getFuncCacheInterval($function) {
$func = $this->getFullyQualified($function);
if (!$this->functionExists($func)) return false;
return $this->config[$func]["cache"]["interval"];
}

function getFuncNeedsRepublish($function) {
$func = $this->getFullyQualified($function);
if (!$this->functionExists($func)) return false;
return $this->config[$func]["cache"]["republish"];
}

/**
  *                                       
  *
  *                                                                                     
  *                                           
  *                                                           
  **/
function getPackageURI($package) {
if (!$this->packageExists($package)) return false;
return $this->packages[$package]["connect"]["uri"];
}

/**
  *                                                                
  *
  *                                                           
  *                                                     
  **/
function getPackageType($package) {
if (!$this->packageExists($package)) return false;
return $this->packages[$package]["connect"]["type"];
}

/**
  *                                                   
  *                                              
  **/
function getPackages() {
return array_keys($this->packages);
}

/**
  *                                                  
  *                                                     
  **/
function getFunctions() {
return array_keys($this->config);
}

/**
  *                                                  
  *
  *                                                                                             
  *                                           
  *                                            
  *                                                                                               
  **/
function getFuncParams($function) {
$func = $this->getFullyQualified($function);
if (!$this->functionExists($func)) return false;
$rs = array();
reset($this->config[$func]["params"]);
while (list($key,$param) = each($this->config[$func]["params"])) {
$rs[$key] = $param["default"];
}
return $rs;
}

/**
  *                                                                   
  *
  *                                                                                            
  *                      
  *                                            
  *                     
  *                                                                     
  **/
function getFuncParamDescription($function, $param) {
$func = $this->getFullyQualified($function);
if (!$this->functionExists($func)) return false;
return (is_array($this->config[$func]["params"][$param]))?$this->config[$func]["params"][$param]["description"]:false;
}

/**
  *                                                                           
  *                                            
  *                                                        
  **/
function getFuncPackage($function) {
$func = $this->getFullyQualified($function);
if (!$this->functionExists($func)) return false;
return $this->config[$func]["package"];
}

/**
  *                                                                    
  *                                            
  *                                    
  **/
function getFuncFile($function) {
$func = $this->getFullyQualified($function);
if (!$this->functionExists($func)) return false;
return $this->config[$func]["file"]["filename"];
}

/**
  *                                                                         
  *                                           
  *                                       
  **/
function getFuncMethod($function) {
$rs = explode(".",$function);
return $rs[count($rs) - 1];
}

/**
  *                                                  
  *                                  
  *                                  
  **/
function hasSignature($function) {
return true;
}

/**
  *                                                        
  *                                                              
  *                                                
  *                                           
  **/
function checkSignature($function) {
if (!$this->hasSignature($function)) return false;
return true;
}

/**
  *              
  **/
function getFuncConfig($function) {
$this->resetError();
//                                         
 $package = $this->getFuncPackage($function);
$params = $this->getFuncParams($function);
$method = $this->getFuncMethod($function);
$file = $this->getFuncFile($function);
$uri = $this->getPackageURI($package);
$type = $this->getPackageType($package);


if (!$uri || !$type) {
$this->error = 2;
$this->errorMsg = "Could not find an uri ($uri) or type ($type) for this function ($function).";
return "";
}

$config = array();
$config["params"] = $params;
$config["method"] = $method;
$config["package"] = $package;
$config["file"] = $file;
$config["uri"] = $uri;
$config["type"] = $type;
return $config;
}

/**
  *                                              
  *
  *                                                                          
  *                                                                             
  *                                     
  * 
  *                                                                      
  *                                                            
  *                                       
  **/
function call($function, $code) {
$this->resetError();

$parser = $this->getXNLParser( $function, $code );
$parser->execute();
if ($parser->getErrorID()) {
$this->error = $parser->getErrorID();
$this->errorMsg = $parser->getErrorMsg();
$res = "";
} else {
$res = $parser->getResult();
}

return $res;
}

/**
  *                                                                                                 
  *                                            
  *                                       
  *                                                  
  **/
function getXNLParser($function, $code) {
$config = $this->getFuncConfig($function);
$parser = new XNLFuncParser($function, $config, $code);
$parser->addParams( $this->params );
return $parser;
}

/**
  *                                      
  *                                                                                
  **/
function getErrorID() {
return $this->error;
}

/**
  *                                                 
  *                                                                                      
  **/
function getErrorMsg() {
return $this->errorMsg;
}

/**
  *                                                        
  **/
function resetError() {
$this->error = 0;
$this->errorMsg = "";
}

/**
  *
  *
  *
  */
function addParams( $params )
{
$this->params = array_merge( $this->params, $params );
}

function addParam( $name, $value )
{
$this->params[$name] = $value;
}
}

class XNLFuncParser extends BaseObject {

var $function;
var $code;
var $config;
var $params;
var $error;
var $errorMsg;
var $result;
var $deps;
var $add_params = array();

function XNLFuncParser($function, $config, $code) {
$this->function = $function;
$this->config = $config;
$this->code = $code;
$this->params = false;
$this->result = "";
$this->deps = array();
}

function addParams( $params )
{
$this->add_params = array_merge( $this->add_params, $params );
}

/**
  *                                                                                    
  *
  *                                                                       
  *                                                                      
  *                                                                                                       
  *                                                             
  **/
function parseFunctionParams($editable = false, $defaults = true) {
$this->params = array();
while (list($param, $default) = each($this->config["params"])) {
$rs = $this->_parseCodeParam($param,($defaults)?$default:"", $editable);
if (!is_false($rs)) $this->params[$param] = $rs;
else $this->params[$param] = "";
}
return $this->params;
}

/**
  *                                                              
  *                         
  *                                                            
  **/
function stripCodeParams($keepEditable = false) {
$code = $this->code;
reset($this->config["params"]);
while (list($cparam, $default) = each($this->config["params"])) {
$lcode = strtolower($code);
$param = strtolower($cparam);
$start = minF(strpos($lcode, "<$param "), strpos($lcode, "<$param>"));
if ($start === false) continue;
$tagend = strpos($lcode, ">", $start + 1);
$parts = explode(" ", trim(substr($lcode, $start + 1, $tagend - $start - 1)));
$endtag = strpos($lcode, "</$param>");
if ($endtag === false) continue;
if ($parts[0] != $param) continue;
unset($parts[0]);
$c = substr($code,0,$start);
if (in_array("edit",$parts)) {
$c .= substr($code, $tagend + 1, $endtag - $tagend - 1);
}
$c .= substr($code, $endtag + strlen("</$param>"));
$code = $c;
}
return $code;
}

/**
  *                                              
  **/
function execute() {
$this->resetError();

$config = $this->config;

//                                                                                    
 ///                                                                  
 switch ($config["type"]) {
case "POST": 
default:     $conn = new XNLPostConnect($this->function, $config);
}
$fetched = $this->parseFunctionParams();
if ( is_array( $this->add_params ) )
$params = array_merge( $fetched, $this->add_params );
else
$params = $fetched;
//                      
 $conn->loadCode($this->code, $params);
$conn->execute();
$tags = array();
$this->deps = array();
if ($conn->getErrorID()) {
$this->error = $conn->getErrorID();
$this->errorMsg = $conn->getErrorMsg();
} else {
$res = $conn->getResult();
$code = urldecode($res["RESULT"]["childs"]["RETURNVALUE"][0]["value"]);
$infos = $res["RESULT"]["childs"]["INFOS"][0]["childs"]["INFO"];
$deps = $res["RESULT"]["childs"]["DEPENDENCIES"][0]["childs"]["DEPENDENCY"];
if ( is_array( $deps ) )
{
foreach ( $deps as $dep )
{
$this->deps[] = array("type"=>$dep["attributes"]["TYPE"], "id"=>$dep["childs"]["ID"][0]["value"], "event"=>(int)$dep["childs"]["EVENT"][0]["value"], "ignore"=>(int)$dep["childs"]["IGNORE"][0]["value"]);
}
}
if ( is_array( $infos ) )
{
node_include("i18n");
foreach ($infos as $info) {
$type = strtolower($info["attributes"]["TYPE"]);
if ($type == "error") {
$tags[] = new NodeException(i18n("Error")." (".$info["childs"]["KEY"][0]["value"]."): ".$info["childs"]["TEXT"][0]["value"],
$info["childs"]["DETAIL"][0]["value"]);
} else if ($type == "warning") {
$tags[] = new NodeException(i18n("Warning")." (".$info["childs"]["KEY"][0]["value"]."): ".$info["childs"]["TEXT"][0]["value"],
$info["childs"]["DETAIL"][0]["value"]);
} else {
$tags[] = new ParserMessage(i18n("Info")." (".$info["childs"]["KEY"][0]["value"]."): ".$info["childs"]["TEXT"][0]["value"],
$info["childs"]["DETAIL"][0]["value"]);
}
}
}
$tags[] = new StringObject($code);
}
$this->result = $tags;

return $tags;

}

/**
  *                                                                                                  
  *                                        
  **/
function getResult() {
return $this->result;
}

function getDependencies() {
return $this->deps;
}

/**
  *                                      
  *                                                                                
  **/
function getErrorID() {
return $this->error;
}

/**
  *                                                 
  *                                                                                      
  **/
function getErrorMsg() {
return $this->errorMsg;
}

/**
  *                                                        
  **/
function resetError() {
$this->error = 0;
$this->errorMsg = "";
}

/**
  *                                                                                   
  *
  *                                    
  *                                               
  *                                                                          
  *                                                                /                
  **/
function _parseCodeParam($param, $default = "", $editonly = false) {
if ($editonly) $default = false;
if (!eregi("<$param( [^>]*)?>(.*)</$param *>",$this->code, $regs)) {
return $default;
}

$params = explode(" ",strtolower(trim($regs[1])));
if ($editonly && !in_array("edit",$params)) {
return $default;
}
$param_value = $regs[2];
return $param_value;
}

}

/**
 *                                         
 *                                                        
 **/
class XNLConnect extends BaseObject {

var $function;
var $code;
var $params;
var $config;
var $type;
var $executed;
var $error;
var $errorMsg;
var $result;

/**
  *                                                                                       
  **/
function XNLConnect($function, $config, $type) {
$this->resetError();
$this->function = $function;
$this->config = $config;
$this->code = "";
$this->params = array();
$this->type = $type;
$this->result = "";
}

/**
  *                                                   
  **/
function loadCode($code, $params) {
$this->code = $code;
$this->params = $params;
}

/**
  *                                                   
  * 
  *                                                                                         
  *                          
  **/
function execute() {
$this->resetError();
$rs = $this->_doFetch();
if (!$this->error) {
$this->result = $rs;
$this->executed = true;
}
}

/**
  *                                                            
  *                               
  **/
function isExecuted() {
return $this->executed;
}

/**
  *                                                             
  *                                    
  **/
function hasResult() {
return ($this->executed && $this->error == 0);
}

/**
  *                                                                                                  
  *                                        
  **/
function getResult() {
return ($this->hasResult())?$this->result:"";
}

/**
  *                                      
  *                                                                                
  **/
function getErrorID() {
return $this->error;
}

/**
  *                                                 
  *                                                                                      
  **/
function getErrorMsg() {
return $this->errorMsg;
}

/**
  *                                                        
  **/
function resetError() {
$this->error = 0;
$this->errorMsg = "";
}

/**
  *                                                                    
  * 
  *                                                                               
  *
  *                               
  **/
function _doFetch() {}
}

/**
 *                                                                                                 
 *                                        
 **/
class XNLPostConnect extends XNLConnect {

function XNLPostConnect($function, $config) {
XNLConnect::XNLConnect($function, $config, "POST");
}

function _doFetch() {
global $DEBUG, $ICO_DEBUG;

//             
 $url = $this->config["uri"] . (strpos($url, "?")?"&":"?") . "method=";
$url .= $this->config["package"].".".$this->config["file"].".".$this->config["method"];

//          
 $curl = new fetchCurl("POST");
$curl->setURL($url);

//              
 reset($this->params);
while(list($param,$value) = each($this->params)) {
if (!is_false($value)) $curl->setPostVar($param, rawurlencode($value));
}
$curl->setPostVar("code", rawurlencode($this->code));

if ($DEBUG["echo"] && $DEBUG["xnlinfo"]) $_t1 = microtime();

$rs = $curl->sendRequest();

if ($DEBUG["echo"] && $DEBUG["xnlinfo"]) $_t2 = microtime();

//                             
 $xmlObj = new XMLParser();
$xmlObj->setToLower(true);
$xmlObj->loadXML($rs);
$xnlArr = $xmlObj->getStruct();

if ($DEBUG["echo"] && $DEBUG["xnlinfo"]) {
$_t3 = microtime();
$_t1 = calctime($_t1, $_t2, 5);
$_t2 = calctime($_t2, $_t3, 5);
node_class( "debugstreaminfo" );
$obj =& new DebugStreamInfo();
$addInfo = "Request: $url\n";
$addInfo .= "----- Postvariables -----\n";
foreach ($this->params as $param=>$value) $addInfo .= "$param: $value\n";
$addInfo .= "----- Code -----\n".$this->code."\n----- Result -----\n".$rs;
$obj->addMessage("debug", $ICO_DEBUG, "Exec XNL Function '".$this->config["package"].".".$this->config["method"]." ($_t1,$_t2)", $addInfo);
}

if (is_Array($xnlArr)) {
//                  
 return $xnlArr;
} elseif (ereg("Parse error:(.* on line [0-9]+).*$",$rs,$regs)) {
//          
 $this->error = 1;
$this->errorMsg = substr($rs, 0, 1024);
return Array();
} else if ($xmlObj->isError()) {
//          
 $this->error = 1;
$this->errorMsg = i18n("Error in result code") ."; ".$xmlObj->getErrorMessage()." (Line ".$xmlObj->getErrorLine().")". htmlentities($rs);
} else {
//                
 $this->error = 1;
$this->errorMsg = i18n("Unknown Error: {0}",0,substr($rs, 0, 1024));
return Array();
}

return Array();

}

}

?>
